@@ -11,6 +11,7 @@ our $MESSAGES = {
     maxItems        => 'Too many items: %3/%4.',
     minItems        => 'Not enough items: %3/%4.',
     uniqueItems     => 'Unique items required.',
+    contains        => 'No items contained.',
   },
   const   => {const => 'Does not match const: %3.'},
   enum    => {enum  => 'Not in enum list: %3.'},
@@ -35,6 +36,7 @@ our $MESSAGES = {
     maxProperties        => 'Too many properties: %3/%4.',
     minProperties        => 'Not enough properties: %3/%4.',
     required             => 'Missing property.',
+    dependencies         => 'Missing property. Dependee: %3.',
   },
   oneOf => {
     all_rules_match => 'All of the oneOf rules match.',
